"use server";

import prisma from "@/lib/prisma";
import { s3 } from "@/lib/s3";
import { DeleteObjectCommand } from "@aws-sdk/client-s3";

export async function deleteImage(imageId: string) {
  const image = await prisma.image.findUnique({
    where: { id: imageId },
    include: {
      variants: true,
      palettes: { include: { items: true } },
      colors: true,
      extractColors: true,
      theme: true,
      metadata: true,
      pixels: true,
      stats: true,
    },
  });

  if (!image) throw new Error("Image not found");

  // Delete S3 objects
  for (const variant of image.variants) {
    await s3.send(new DeleteObjectCommand({
      Bucket: "felliesartapp",
      Key: variant.s3Key,
    }));
  }

  // Delete image variants
  await prisma.imageVariant.deleteMany({ where: { imageId } });

  // Delete extract colors
  await prisma.extractColor.deleteMany({ where: { imageId } });

  // Delete image colors
  await prisma.imageColor.deleteMany({ where: { imageId } });

  // Delete palettes (and items only if no other image uses them)
  const palettes = await prisma.colorPalette.findMany({
    where: { images: { some: { id: imageId } } },
    include: { images: { select: { id: true } }, items: true }
  });

  for (const palette of palettes) {
    if (palette.images.length === 1 && palette.images[0].id === imageId) {
      await prisma.colorPaletteItem.deleteMany({ where: { colorPaletteId: palette.id } });
      await prisma.colorPalette.delete({ where: { id: palette.id } });
    }
  }

  // Delete metadata-related entries
  await prisma.imageMetadata.deleteMany({ where: { imageId } });
  await prisma.imageStats.deleteMany({ where: { imageId } });
  await prisma.pixelSummary.deleteMany({ where: { imageId } });
  await prisma.themeSeed.deleteMany({ where: { imageId } });

  // Finally delete the image
  await prisma.image.delete({ where: { id: imageId } });

  return { success: true };
}