"use server"

import prisma from "@/lib/prisma";
import { s3 } from "@/lib/s3";
import { imageUploadSchema } from "@/schemas/images/imageSchema";
import { determineBestWatermarkPosition } from "@/utils/determineWatermarkPosition";
import { PutObjectCommand } from "@aws-sdk/client-s3";
import path from "path";
import sharp from "sharp";
import { v4 as uuidv4 } from 'uuid';
import * as z from "zod/v4";

export async function uploadImage(values: z.infer<typeof imageUploadSchema>) {
  const imageFile = values.file[0];
  const imageName = values.imageName;

  if (!(imageFile instanceof File)) {
    console.log("No image or invalid type");
    return null;
  }

  if (!imageName) {
    console.log("No name for the image provided");
    return null;
  }

  const fileName = imageFile.name;
  const fileType = imageFile.type;
  const fileSize = imageFile.size;
  const lastModified = new Date(imageFile.lastModified);
  const year = lastModified.getUTCFullYear();
  const month = lastModified.getUTCMonth() + 1;

  const fileKey = uuidv4();

  const arrayBuffer = await imageFile.arrayBuffer();
  const buffer = Buffer.from(arrayBuffer);

  const originalKey = `original/${fileKey}.webp`;
  const watermarkedKey = `watermarked/${fileKey}.webp`;
  const resizedKey = `resized/${fileKey}.webp`;
  const thumbnailKey = `thumbnails/${fileKey}.webp`;

  const sharpData = sharp(buffer);
  const metadata = await sharpData.metadata();
  const stats = await sharpData.stats();
  
  //--- Original file
  await s3.send(
    new PutObjectCommand({
      Bucket: "felliesartapp",
      Key: originalKey,
      Body: buffer,
      ContentType: "image/" + metadata.format,
    })
  );
  //--- Watermarked file
  const watermarkPath = path.join(process.cwd(), 'public/watermark/fellies-watermark.svg');
  const watermarkWidth = Math.round(metadata.width * 0.25);
  const gravity = await determineBestWatermarkPosition(buffer);
  const watermarkBuffer = await sharp(watermarkPath)
    .resize({ width: watermarkWidth })
    .png()
    .toBuffer();
  const watermarkedBuffer = await sharp(buffer)
    .composite([{ input: watermarkBuffer, gravity, blend: 'over' }])
    .toFormat('webp')
    .toBuffer()
  const watermarkedMetadata = await sharp(watermarkedBuffer).metadata();
  await s3.send(
    new PutObjectCommand({
      Bucket: "felliesartapp",
      Key: watermarkedKey,
      Body: watermarkedBuffer,
      ContentType: "image/" + watermarkedMetadata.format,
    })
  );
  //--- Resized file
  const resizedWidth = Math.min(watermarkedMetadata.width || 400, 400);
  const resizedBuffer = await sharp(watermarkedBuffer)
    .resize({ width: resizedWidth, withoutEnlargement: true })
    .toFormat('webp')
    .toBuffer();
  const resizedMetadata = await sharp(resizedBuffer).metadata();
  await s3.send(
    new PutObjectCommand({
      Bucket: "felliesartapp",
      Key: resizedKey,
      Body: resizedBuffer,
      ContentType: "image/" + resizedMetadata.format,
    })
  );
  //--- Thumbnail file
  const thumbnailWidth = Math.min(watermarkedMetadata.width || 200, 200);
  const thumbnailBuffer = await sharp(watermarkedBuffer)
    .resize({ width: thumbnailWidth, withoutEnlargement: true })
    .toFormat('webp')
    .toBuffer();
  const thumbnailMetadata = await sharp(thumbnailBuffer).metadata();
  await s3.send(
    new PutObjectCommand({
      Bucket: "felliesartapp",
      Key: thumbnailKey,
      Body: thumbnailBuffer,
      ContentType: "image/" + thumbnailMetadata.format,
    })
  );

  const image = await prisma.image.create({
    data: {
      imageName,
      fileKey,
      originalFile: fileName,
      uploadDate: new Date(),

      creationDate: lastModified,
      creationMonth: month,
      creationYear: year,
      fileType: fileType,
      fileSize: fileSize
    },
  });

  await prisma.imageMetadata.create({
    data: {
      imageId: image.id,
      format: metadata.format || "unknown",
      width: metadata.width || 0,
      height: metadata.height || 0,
      space: metadata.space || "unknown",
      channels: metadata.channels || 0,
      depth: metadata.depth || "unknown",
      density: metadata.density ?? undefined,
      bitsPerSample: metadata.bitsPerSample ?? undefined,
      isProgressive: metadata.isProgressive ?? undefined,
      isPalette: metadata.isPalette ?? undefined,
      hasProfile: metadata.hasProfile ?? undefined,
      hasAlpha: metadata.hasAlpha ?? undefined,
      autoOrientW: metadata.autoOrient?.width ?? undefined,
      autoOrientH: metadata.autoOrient?.height ?? undefined,
    },
  });

  await prisma.imageStats.create({
    data: {
      imageId: image.id,
      isOpaque: stats.isOpaque,
      entropy: stats.entropy,
      sharpness: stats.sharpness,
      dominantR: stats.dominant.r,
      dominantG: stats.dominant.g,
      dominantB: stats.dominant.b,
    },
  });

  await prisma.imageVariant.createMany({
    data: [
      {
        s3Key: originalKey,
        type: "original",
        height: metadata.height,
        width: metadata.width,
        fileExtension: metadata.format,
        mimeType: "image/" + metadata.format,
        sizeBytes: metadata.size,
        imageId: image.id
      },
      {
        s3Key: watermarkedKey,
        type: "watermarked",
        height: watermarkedMetadata.height,
        width: watermarkedMetadata.width,
        fileExtension: watermarkedMetadata.format,
        mimeType: "image/" + watermarkedMetadata.format,
        sizeBytes: watermarkedMetadata.size,
        imageId: image.id
      },
      {
        s3Key: resizedKey,
        type: "resized",
        height: resizedMetadata.height,
        width: resizedMetadata.width,
        fileExtension: resizedMetadata.format,
        mimeType: "image/" + resizedMetadata.format,
        sizeBytes: resizedMetadata.size,
        imageId: image.id
      },
      {
        s3Key: thumbnailKey,
        type: "thumbnail",
        height: thumbnailMetadata.height,
        width: thumbnailMetadata.width,
        fileExtension: thumbnailMetadata.format,
        mimeType: "image/" + thumbnailMetadata.format,
        sizeBytes: thumbnailMetadata.size,
        imageId: image.id
      }
    ],
  });

  return image
}