"use client"

import { updateAlbum } from "@/actions/albums/updateAlbum";
import { Button } from "@/components/ui/button";
import { Form, FormControl, FormDescription, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { Album, Gallery, Image } from "@/generated/prisma";
import { albumSchema } from "@/schemas/albums/albumSchema";
import { zodResolver } from "@hookform/resolvers/zod";
import NextImage from "next/image";
import { useRouter } from "next/navigation";
import { useForm } from "react-hook-form";
import { toast } from "sonner";
import * as z from "zod/v4";

type AlbumWithItems = Album & {
  images: Image[],
  coverImage: Image | null
}

export default function EditAlbumForm({ album, galleries }: { album: AlbumWithItems, galleries: Gallery[] }) {
  const router = useRouter();
  const form = useForm<z.infer<typeof albumSchema>>({
    resolver: zodResolver(albumSchema),
    defaultValues: {
      name: album.name,
      slug: album.slug,
      description: album.description || "",
      galleryId: album.galleryId || "",
      coverImageId: album.coverImage?.id || "",
    },
  })

  async function onSubmit(values: z.infer<typeof albumSchema>) {
    const updatedAlbum = await updateAlbum(values, album.id)
    if (updatedAlbum) {
      toast.success("Album updated")
      router.push(`/albums`)
    }
  }

  const selectedImage = album.images.find(img => img.id === form.watch("coverImageId"));

  return (
    <div className="flex flex-col gap-8">
      <Form {...form}>
        <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
          <FormField
            control={form.control}
            name="name"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Album name</FormLabel>
                <FormControl>
                  <Input placeholder="Album name" {...field} />
                </FormControl>
                <FormDescription>
                  This is your public display name.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="slug"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Album slug</FormLabel>
                <FormControl>
                  <Input placeholder="Album slug" {...field} />
                </FormControl>
                <FormDescription>
                  Will be used for the navigation.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="description"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Album description (optional)</FormLabel>
                <FormControl>
                  <Input placeholder="Album description" {...field} />
                </FormControl>
                <FormDescription>
                  Description of the Album.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="galleryId"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Gallery</FormLabel>
                <Select onValueChange={field.onChange} defaultValue={field.value}>
                  <FormControl>
                    <SelectTrigger>
                      <SelectValue placeholder="Select a gallery" />
                    </SelectTrigger>
                  </FormControl>
                  <SelectContent>
                    {galleries.map((gallery) => (
                      <SelectItem key={gallery.id} value={gallery.id}>
                        {gallery.name}
                      </SelectItem>
                    ))}
                  </SelectContent>
                </Select>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="coverImageId"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Cover Image</FormLabel>
                <Select onValueChange={field.onChange} value={field.value}>
                  <FormControl>
                    <SelectTrigger>
                      <SelectValue placeholder="Select a cover image" />
                    </SelectTrigger>
                  </FormControl>
                  <SelectContent>
                    {album.images.map((image) => (
                      <SelectItem key={image.id} value={image.id}>
                        {image.imageName}
                      </SelectItem>
                    ))}
                  </SelectContent>
                </Select>
                <FormDescription>
                  Optional cover image shown in album previews.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          {selectedImage?.fileKey && (
            <div className="mt-2">
              <p className="text-sm text-muted-foreground mb-1">Cover preview:</p>
              <NextImage
                src={`/api/image/thumbnails/${selectedImage.fileKey}.webp`}
                width={128}
                height={128}
                alt="Selected cover"
                className="w-32 h-auto rounded border"
              />
            </div>
          )}
          <div className="flex flex-col gap-4">
            <Button type="submit">Submit</Button>
            <Button type="reset" variant="secondary" onClick={() => router.back()}>Cancel</Button>
          </div>
        </form>
      </Form>
    </div>
  );
}