"use client"

import { createArtist } from "@/actions/artists/createArtist";
import { Button } from "@/components/ui/button";
import { Form, FormControl, FormDescription, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { artistSchema } from "@/schemas/artists/artistSchema";
import { getSocialIcon } from "@/utils/socialIconMap";
import { zodResolver } from "@hookform/resolvers/zod";
import { useRouter } from "next/navigation";
import { useFieldArray, useForm } from "react-hook-form";
import { toast } from "sonner";
import * as z from "zod/v4";

export default function CreateArtistForm() {
  const router = useRouter();
  const form = useForm<z.infer<typeof artistSchema>>({
    resolver: zodResolver(artistSchema),
    defaultValues: {
      displayName: "",
      slug: "",
      nickname: "",
      socials: [],
    },
  })

  const { fields, append, remove } = useFieldArray({
    control: form.control,
    name: "socials",
  });

  const watchSocials = form.watch("socials");

  async function onSubmit(values: z.infer<typeof artistSchema>) {
    const artist = await createArtist(values)
    if (artist) {
      toast.success("Artist created")
      router.push(`/artists`)
    }
  }

  return (
    <Form {...form}>
      <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
        <FormField
          control={form.control}
          name="displayName"
          render={({ field }) => (
            <FormItem>
              <FormLabel>Artist name</FormLabel>
              <FormControl>
                <Input placeholder="Artist name" {...field} />
              </FormControl>
              <FormDescription>
                This is your public display name.
              </FormDescription>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          control={form.control}
          name="slug"
          render={({ field }) => (
            <FormItem>
              <FormLabel>Artist slug</FormLabel>
              <FormControl>
                <Input placeholder="Artist slug" {...field} />
              </FormControl>
              <FormDescription>
                Will be used for the navigation.
              </FormDescription>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          control={form.control}
          name="nickname"
          render={({ field }) => (
            <FormItem>
              <FormLabel>Artist nickname</FormLabel>
              <FormControl>
                <Input placeholder="Artist nickname" {...field} />
              </FormControl>
              <FormDescription>
                Nickname of the artist.
              </FormDescription>
              <FormMessage />
            </FormItem>
          )}
        />

        {/* Socials section */}
        <div className="space-y-4">
          <FormLabel>Social Links</FormLabel>
          {fields.map((field, index) => (
            <div key={field.id} className="flex flex-col gap-2 border rounded p-4">
              <div className="flex items-center gap-2">
                {getSocialIcon(watchSocials?.[index]?.platform ?? "")}
                <FormField
                  control={form.control}
                  name={`socials.${index}.platform`}
                  render={({ field }) => (
                    <FormItem className="w-full">
                      <FormLabel>Platform</FormLabel>
                      <FormControl>
                        <Input placeholder="e.g. telegram" {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
              </div>
              <FormField
                control={form.control}
                name={`socials.${index}.handle`}
                render={({ field }) => (
                  <FormItem>
                    <FormLabel>Handle</FormLabel>
                    <FormControl>
                      <Input placeholder="@username" {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                control={form.control}
                name={`socials.${index}.link`}
                render={({ field }) => (
                  <FormItem>
                    <FormLabel>Link</FormLabel>
                    <FormControl>
                      <Input placeholder="https://example.com/username" {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                control={form.control}
                name={`socials.${index}.isPrimary`}
                render={({ field }) => (
                  <FormItem className="flex items-center gap-2">
                    <input
                      type="radio"
                      checked={field.value}
                      onChange={
                        () =>
                          form.setValue(
                            "socials",
                            (watchSocials ?? []).map((s, i) => ({
                              ...s,
                              isPrimary: i === index,
                            }))
                          )
                      }
                    />
                    <FormLabel className="m-0 text-sm">Set as primary</FormLabel>
                  </FormItem>
                )}
              />
              <Button
                type="button"
                variant="ghost"
                onClick={() => remove(index)}
              >
                Remove Social
              </Button>
            </div>
          ))}
          <Button
            type="button"
            variant="outline"
            onClick={() => append({ platform: "", handle: "", link: "", isPrimary: false })}
          >
            + Add Social
          </Button>
        </div>

        <div className="flex flex-col gap-4">
          <Button type="submit">Submit</Button>
          <Button type="reset" variant="secondary" onClick={() => router.back()}>Cancel</Button>
        </div>
      </form>
    </Form>
  );
}