"use client"

import { deleteCategory } from "@/actions/categories/deleteCategory";
import { updateCategory } from "@/actions/categories/updateCategory";
import { Button } from "@/components/ui/button";
import { Form, FormControl, FormDescription, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { Category } from "@/generated/prisma";
import { categorySchema } from "@/schemas/categories/categorySchema";
import { zodResolver } from "@hookform/resolvers/zod";
import { useRouter } from "next/navigation";
import { useForm } from "react-hook-form";
import { toast } from "sonner";
import * as z from "zod/v4";

export default function EditCategoryForm({ category }: { category: Category }) {
  const router = useRouter();
  const form = useForm<z.infer<typeof categorySchema>>({
    resolver: zodResolver(categorySchema),
    defaultValues: {
      name: category.name,
      description: category.description || "",
    },
  })

  async function onSubmit(values: z.infer<typeof categorySchema>) {
    const updatedCategory = await updateCategory(values, category.id)
    if (updatedCategory) {
      toast.success("Category updated")
      router.push(`/categories`)
    }
  }

  return (
    <div className="flex flex-col gap-8">
      <Form {...form}>
        <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
          <FormField
            control={form.control}
            name="name"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Category name</FormLabel>
                <FormControl>
                  <Input placeholder="Category name" {...field} />
                </FormControl>
                <FormDescription>
                  This is your public display name.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="description"
            render={({ field }) => (
              <FormItem>
                <FormLabel>Category description (optional)</FormLabel>
                <FormControl>
                  <Input placeholder="Category description" {...field} />
                </FormControl>
                <FormDescription>
                  Description of the Category.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />
          <div className="flex flex-col gap-4">
            <Button type="submit">Submit</Button>
            <Button type="reset" variant="secondary" onClick={() => router.back()}>Cancel</Button>
          </div>
        </form>
      </Form>
      <div>
        <Button
          type="button"
          variant="destructive"
          onClick={async () => {
            await deleteCategory(category.id);
            toast.success("Category deleted");
            router.push("/categories");
          }}
        >
          Delete Category
        </Button>
      </div>
    </div>
  );
}